{ config, pkgs, lib, ... }:
{
  # Bootloader
  boot.loader.systemd-boot.enable = true;
  boot.loader.efi.canTouchEfiVariables = true;
  boot.loader.efi.efiSysMountPoint = "/boot/efi";

  services.displayManager.autoLogin.enable = true;
  services.displayManager.autoLogin.user = "pbsds";
  # tmp: https://github.com/NixOS/nixpkgs/issues/103746#issuecomment-945091229
  systemd.services."getty@tty1".enable = false;
  systemd.services."autovt@tty1".enable = false;

  boot.kernelPackages = pkgs.linuxPackages_latest; # amdgpu audio fix when at 4k@60fps

  imports = [
    ./hardware-configuration.nix
    ../../../profiles/sshd.nix
    #../../../profiles/oci/podman.nix
    #../../../profiles/oci/docker.nix

    #../../../profiles/vpn-pbsds/tailscale.nix

    ../../../users/pbsds
    ../../../users/pbsds/syncthing.nix

    ../../../profiles/mounts/freon-nfs.nix
    ../../../profiles/mounts/reidun-nfs.nix
    /* ../../../profiles/mounts/meconium-nfs.nix */
    /* ../../../profiles/mounts/fridge-nfs.nix */

    ../../../profiles/shell.nix

    ../../../profiles/desktop
    ../../../profiles/desktop/gnome # configures gdm
    #../../../profiles/desktop/sound/alsa.nix
    #../../../profiles/desktop/sound/pulseaudio.nix
    ../../../profiles/desktop/sound/pipewire.nix

    ../../../profiles/desktop/steam.nix
    ../../../profiles/desktop/flatpak.nix

    ../../../profiles/known-hosts.nix
    #../../../profiles/domeneshop-dyndns.nix # handled by noximilien
  ];

  nix.settings.max-jobs = 1; # this host do be stinky

  networking.firewall.allowedTCPPorts = [ 57621 ]; # spotify local discovery

  #networking.wireguard.interfaces."wg0".ips = [ "172.22.48.4/24" ]; # fyrkat


  # Networking
  networking.networkmanager.enable = true;
  #networking.wireless.enable = true;  # Enables wireless support via wpa_supplicant.
  networking.interfaces.eno1.ipv4.addresses = [
    { address = "192.168.1.8"; prefixLength = 24; }
  ];
  networking.defaultGateway.address = "192.168.1.254";
  networking.defaultGateway.interface = "eno1";
  networking.nameservers = [
    "192.168.1.254"
    "8.8.8.8"
    "1.1.1.1"
  ];
  #networking.useDHCP = true;
  #TODO: avahi? resolved? https://git.pvv.ntnu.no/Drift/pvv-nixos-config/src/main/base.nix#L15-L18

  # TODO: remove? Move?
  programs.dconf.enable = true;

  # Enable touchpad support (enabled default in most desktopManager).
  # services.xserver.libinput.enable = true;
}
